unit Main; {Fractal Landscapes 3.0 - Copyright  1987..1997, Jon Shemitz}

{ $define RDTSC} {displays timing info - ONLY on Pentia}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, Gauges,
  Printers, Clipbrd,
  Global, Options, Database, Display, About, KbdHelp
  {$ifdef RDTSC} ,Pentium {$endif}
  ;

type
  TFractalLandscape = class(TForm)
    MainMenu: TMainMenu;
    FileNewItem: TMenuItem;
    FileOpenItem: TMenuItem;
    FileSaveAsItem: TMenuItem;
    FilePrintItem: TMenuItem;
    FilePrintSetupItem: TMenuItem;
    FileExitItem: TMenuItem;
    EditCopyItem: TMenuItem;
    HelpAboutItem: TMenuItem;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    PrintDialog: TPrintDialog;
    PrintSetupDialog: TPrinterSetupDialog;
    SpeedBar: TPanel;
    SpeedButton1: TSpeedButton;  { &New }
    SpeedButton2: TSpeedButton;  { &Open... }
    SpeedButton3: TSpeedButton;  { Save &As... }
    SpeedButton4: TSpeedButton;  { &Print... }
    SpeedButton5: TSpeedButton;  { &Copy }
    SpeedButton6: TSpeedButton;
    Image: TImage;
    StatusPnl: TPanel;
    PlyPnl: TPanel;
    ModePnl: TPanel;
    DebugPnl: TPanel;
    Options1: TMenuItem;
    DisplayOptions1: TMenuItem;
    GaugePnl: TPanel;
    DrawProgress: TGauge;
    OutlineBtn: TSpeedButton;
    FillBtn: TSpeedButton;
    RenderBtn: TSpeedButton;
    Ply1: TSpeedButton;
    Ply2: TSpeedButton;
    Ply3: TSpeedButton;
    Ply4: TSpeedButton;
    Ply5: TSpeedButton;
    Ply6: TSpeedButton;
    Ply7: TSpeedButton;
    Ply8: TSpeedButton;
    SealevelPnl: TPanel;
    SeaLevelSldr: TScrollBar;
    SeaLevelLbl: TLabel;
    KbdCmdsItem: TMenuItem;  { &About... }
    procedure FileNew(Sender: TObject);
    procedure FileOpen(Sender: TObject);
    procedure FileSaveAs(Sender: TObject);
    procedure FilePrint(Sender: TObject);
    procedure FilePrintSetup(Sender: TObject);
    procedure FileExit(Sender: TObject);
    procedure EditCopy(Sender: TObject);
    procedure HelpAbout(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure DisplayOptions1Click(Sender: TObject);
    procedure ModeBtnClick(Sender: TObject);
    procedure PlyBtnClick(Sender: TObject);
    procedure SealevelPnlResize(Sender: TObject);
    procedure SeaLevelSldrChange(Sender: TObject);
    procedure SeaLevelSldrScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure KbdCmdsItemClick(Sender: TObject);

  private
    Generated: boolean; {Initialized to False}

    TotalTriangles,
    TrianglesDrawn: LongInt;

    function GetPalette: hPalette; override;

    procedure DrawLandscape(Canvas: TCanvas; Height, Width: integer; Tag: string);

  public
    procedure SetPlys(NewPlys: integer);
    procedure SetDrawMode(NewDrawMode: DrawModes);

    procedure GenerateLandscape;
    procedure ShowLandscape;

    procedure DrewSomeTriangles(N: word);
  end;

var
  FractalLandscape: TFractalLandscape;

implementation

{$R *.DFM}

{windows management}

  procedure TFractalLandscape.SetPlys(NewPlys: integer);
  var
    Idx: integer;
  begin
    Plys := NewPlys;
    PlyPnl.Caption := IntToStr(NewPlys) + ' ply' +
                                          Copy('s', 1, 255 * ord(NewPlys > 1));
    for Idx := 0 to ComponentCount - 1 do
      if Components[Idx] is TSpeedButton then
        with TSpeedButton(Components[Idx]) do
          if GroupIndex = 1 then if Tag = NewPlys then Down := True;
  end;

  function InitPlys: integer; near;
  var
    R, E: integer;
  begin
    Result := 7;
    if ParamCount >= 1 then
      begin
      Val(ParamStr(1), R, E);
      if (E = 0) and (R >= 1) and (R <= MaxPlys) then Result := R;
      end;
  end;


procedure TFractalLandscape.FormCreate(Sender: TObject);
var
  Gray,
  GrayLevel: integer;
  LogPalette: record
            case byte of
              1: (Hdr:  TLogPalette);
              2: (Version: word;
                  Entries: word;
                  Color:   array[0..ShadesOfGray-1] of TPaletteEntry);
            end;

begin
  {$ifNdef Debug} Randomize; {$endif}
  SetPlys(InitPlys);
  Image.Picture.Bitmap := TBitmap.Create;
  LogPalette.Version := $0300;
  LogPalette.Entries := ShadesOfGray;
  for Gray := 1 to ShadesOfGray do
    begin
    GrayLevel := Gray * (256 div ShadesOfGray) - 1;
    with LogPalette.Color[Gray - 1] do
      begin
      peRed   := GrayLevel;
      peGreen := GrayLevel;
      peBlue  := GrayLevel;
      peFlags := 0;
      end;
    end;
  Image.Picture.Bitmap.Palette := CreatePalette(LogPalette.Hdr);
end;

function TFractalLandscape.GetPalette: hPalette;
begin
  Result := Image.Picture.Bitmap.Palette;
end;

procedure TFractalLandscape.FormResize(Sender: TObject);
begin
  DebugPnl.Width := StatusPnl.ClientWidth - DebugPnl.Left - 8;
  DisplayHeight := Image.Height;
  DisplayWidth  := Image.Width;
  with Image.Picture.Bitmap do
    begin
    Height := DisplayHeight;
    Width  := DisplayWidth;
    end;
  Application.ProcessMessages;
  if not Generated then GenerateLandscape; {Form creation}
  if Generated then ShowLandscape;
end;

procedure TFractalLandscape.SetDrawMode(NewDrawMode: DrawModes);
begin
  DrawMode := NewDrawMode;
  case DrawMode of
    dmOutline: begin ModePnl.Caption := 'Outline Mode';
                     OutlineBtn.Down := True; end;
    dmFill:    begin ModePnl.Caption := 'Fill Mode';
                     FillBtn.Down := True; end;
    dmRender:  begin ModePnl.Caption := 'Render Mode';
                     RenderBtn.Down := True; end;
    end;
end;

{fractal generation}

procedure TFractalLandscape.GenerateLandscape;
begin
  PlyPnl.Caption := IntToStr(Plys) + ' ply' +
                                     Copy('s', 1, 255 * ord(Plys > 1));
  DebugPnl.Caption := 'Generating'; Application.ProcessMessages;
  ResetDB;
  FractureTriangle(A, B, C, Plys);
  Generated := True;
end;

{fractal display}

procedure TFractalLandscape.DrawLandscape( Canvas: TCanvas;
                                           Height, Width: integer;
                                           Tag: string);
{$ifdef RDTSC}
var
  StartTicks: comp;
{$endif}
begin
  {$ifdef RDTSC}
    startticks := RDTSC;
  {$endif}
  Screen.Cursor := crHourGlass;
  DebugPnl.Caption := Tag;
  Application.ProcessMessages;

  TotalTriangles := Longint(1) shl ((Plys - 1) * 2); {4 per ply}
  TrianglesDrawn := 0;
  DrawProgress.Progress := 0;
  DrawProgress.Visible := True;
  DisplayHeight  := Height;
  DisplayWidth   := Width;
  DrawTriangle(Canvas, A, B, C, Plys, True);
  DrawVerticals(Canvas);
  DrawProgress.Visible := False;

  DebugPnl.Caption := '';
  {$ifdef Debug}
    DebugPnl.Caption := DebugString;
  {$endif}
  Screen.Cursor := crDefault;
  {$ifdef RDTSC}
    DebugPnl.Caption := CompToStr(RDTSC - StartTicks);
  {$endif}
end;

procedure TFractalLandscape.ShowLandscape;
var
  Redraw: boolean;
begin
  if DrawingNow {Call ProcessMessages (indirectlyt) so can interrupt self}
    then AbortDraw := True
    else try
           repeat
             Redraw := False;
             try
               AbortDraw  := False;
               DrawingNow := True;
               with Image.Picture.Bitmap.Canvas do {Clear the canvas}
                 begin
                 Pen.Color   := clBtnFace;
                 Brush.Color := clBtnFace;
                 Brush.Style := bsSolid;
                 Rectangle(0, 0, DisplayWidth, DisplayHeight);
                 Pen.Color   := clBlack;
                 end;
               ScreenColors;
               DrawLandscape( Image.Picture.Bitmap.Canvas,
                              Image.ClientHeight, Image.ClientWidth, 'Drawing');
             except
               on EAbortedDrawing do Redraw := True;
             end;
           until not Redraw;
         finally
           DrawingNow := False;
         end;
end;

procedure TFractalLandscape.DrewSomeTriangles(N: word);
var
  NewProgress: integer;
const
  Delta: array[DrawRates] of integer = (1, 5, 10);
begin
  Inc(TrianglesDrawn, N);
  NewProgress := (TrianglesDrawn * 100) div TotalTriangles;
  if NewProgress - DrawProgress.Progress >= Delta[DrawRate] then
    begin
    DrawProgress.Progress := NewProgress;
    Application.ProcessMessages; 
    end;
end;

{menu/toolbar fns}

{file}

  procedure TFractalLandscape.FileNew(Sender: TObject);
  begin
    GenerateLandscape;
    ShowLandscape;
  end;

  procedure TFractalLandscape.FileOpen(Sender: TObject);
  begin
    if OpenDialog.Execute and LoadLandscape(OpenDialog.FileName) then
      begin
      SetPlys(Plys);
      ShowLandscape;
      end;
  end;

  procedure TFractalLandscape.FileSaveAs(Sender: TObject);
  var
    Extension: string[5];
  begin
    if SaveDialog.Execute then
    begin
    Extension := UpperCase(ExtractFileExt(SaveDialog.FileName));
    if Extension = '.FL'
      then SaveLandscape(SaveDialog.FileName)
      else if Extension = '.BMP'
        then Image.Picture.SaveToFile(SaveDialog.FileName)
        else MessageDlg('Can''t save ' + Extension + ' files.', mtWarning, [mbOK], 0);
    end;
  end;

  procedure TFractalLandscape.FilePrint(Sender: TObject);
  begin
    if PrintDialog.Execute then
    begin
    Printer.BeginDoc;
    PrinterColors;
    DrawLandscape( Printer.Canvas,
                   Printer.PageHeight, Printer.PageWidth, 'Printing');
    Printer.EndDoc;
    end;
  end;

  procedure TFractalLandscape.FilePrintSetup(Sender: TObject);
  begin
    PrintSetupDialog.Execute;
  end;

  procedure TFractalLandscape.FileExit(Sender: TObject);
  begin
    Close;
  end;

{edit}

  procedure TFractalLandscape.EditCopy(Sender: TObject);
  begin
    Clipboard.Assign(Image.Picture.Bitmap);
  end;

{options}

  procedure TFractalLandscape.DisplayOptions1Click(Sender: TObject);
  begin
    OptionsDlg.Show;
  end;


{help}

  procedure TFractalLandscape.KbdCmdsItemClick(Sender: TObject);
  begin
    KeyboardHelp.Show;
  end;

  procedure TFractalLandscape.HelpAbout(Sender: TObject);
  begin
    AboutBox.Show;
  end;

{other user commands}

procedure TFractalLandscape.FormKeyPress(Sender: TObject; var Key: Char);
var
  Redraw,
  Regen: boolean;
var
  NewPlys:     integer;
  NewDrawMode: DrawModes;
begin
  Redraw      := False;
  Regen       := False;
  NewPlys     := Plys;
  NewDrawmode := DrawMode;
  case UpCase(Key) of
    '+': if NewPlys < MaxPlys then Inc(NewPlys);
    '-': if NewPlys > 1 then       Dec(NewPlys);
    '1'..char(MaxPlys + ord('0')):
	 if NewPlys <> ord(Key) - ord('0') then NewPlys := ord(Key) - ord('0');
    ' ': Regen := True;
    'O': NewDrawMode := dmOutline;
    'F': NewDrawMode := dmFill;
    'R': NewDrawMode := dmRender;
    end;
  if (NewDrawMode <> DrawMode) or (NewPlys <> Plys) then
    OptionsDlg.UpdateOptions(NewDrawMode, DrawRate, NewPlys);
  if Regen or (NewPlys <> Plys) then
    begin
    SetPlys(NewPlys);
    GenerateLandscape;
    Redraw := True;
    end;
  if Redraw or (Drawmode <> NewDrawmode) then
    begin
    SetDrawMode(NewDrawMode);
    ShowLandscape;
    end;
end;

procedure TFractalLandscape.ModeBtnClick(Sender: TObject);
var
  NewMode: DrawModes;
begin
  NewMode := DrawModes((Sender as TSpeedButton).Tag);
  if DrawMode <> NewMode then
    begin
    SetDrawMode(NewMode);
    ShowLandscape;
    end;
end;

procedure TFractalLandscape.PlyBtnClick(Sender: TObject);
var
  NewPlys: integer;
begin
  NewPlys := (Sender as TSpeedButton).Tag;
  if NewPlys <> Plys then
    begin
    SetPlys(NewPlys);
    GenerateLandscape;
    ShowLandscape;
    end;
end;

procedure TFractalLandscape.SealevelPnlResize(Sender: TObject);
begin
  SeaLevelSldr.Height := (SeaLevelPnl.Height - SeaLevelSldr.Top) -
                         SeaLevelLbl.Top;  
end;

procedure TFractalLandscape.SeaLevelSldrChange(Sender: TObject);
begin
(*  SeaLevel := - SeaLevelSldr.Position;
  PostMessage(Handle, wm_ShowLandscape, 0, 0);*)
end;

procedure TFractalLandscape.SeaLevelSldrScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  if (ScrollCode in [ {scLineUp, scLineDown, scPageUp, scPageDown,}
                      scPosition, {scTop, scBottom,} scEndScroll ])
     and
     (SeaLevel <> - SeaLevelSldr.Position) then
    begin
    SeaLevel := - SeaLevelSldr.Position; 
    ShowLandscape;
    end;
end;

end.
